@@ -29,6 +29,11 @@
 #include <glib/gi18n.h>
 #include "koto-date-parser.h"
 
+/*
+ * This entire file is replaced with a stub if we don't have GRegex
+ */
+#if HAVE_DECL_G_REGEX_NEW
+
 /* Set this to 1 to get lots of debugging output */
 #define DEBUG 0
 
@@ -45,17 +50,17 @@ static void
 populate_hash (GHashTable *hash, const char **aliases, int count)
 {
   int i;
-  
+
   g_assert (hash);
   g_assert (aliases);
   g_assert (count);
 
   for (i = 0; i < count; i++) {
     char **tokens, **s;
-    
+
     if (aliases[i] == NULL)
       continue;
-    
+
     tokens = g_strsplit (_(aliases[i]), "|", 0);
     for (s = tokens; *s; s++) {
       g_hash_table_insert (hash, g_utf8_casefold (*s, -1), GINT_TO_POINTER (i));
@@ -68,7 +73,10 @@ static void
 init_days (void)
 {
   const char *aliases[] = {
-    NULL, /* This means that the indexes match GDateWeekday */
+    /* This means that the indexes match GDateWeekday */
+    NULL,
+    /* Translators: these strings should have every common form for weekdays
+       including abbreviations, separated by | */
     N_("monday|mon"),
     N_("tuesday|tue"),
     N_("wednesday|wed"),
@@ -77,7 +85,7 @@ init_days (void)
     N_("saturday|sat"),
     N_("sunday|sun"),
   };
-  
+
   day_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   populate_hash (day_hash, aliases, G_N_ELEMENTS (aliases));
 }
@@ -86,7 +94,10 @@ static void
 init_months (void)
 {
   const char *aliases[] = {
-    NULL, /* This means that the indexes match GDateMonth */
+    /* This means that the indexes match GDateMonth */
+    NULL,
+    /* Translators: these strings should have every common form for month names
+       including abbreviations, separated by | */
     N_("january|jan"),
     N_("february|feb"),
     N_("march|mar"),
@@ -100,7 +111,7 @@ init_months (void)
     N_("november|nov"),
     N_("december|dec"),
   };
-  
+
   month_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   populate_hash (month_hash, aliases, G_N_ELEMENTS (aliases));
 }
@@ -111,7 +122,7 @@ convert_str_weekday (const char *s)
   char *folded;
   GDateWeekday day;
   g_assert (s);
-  
+
   folded = g_utf8_casefold (s, -1);
 #if DEBUG
   g_printerr ("Converting '%s' to a week day\n", folded);
@@ -152,7 +163,7 @@ today (void)
 #if BUILD_TESTS
   g_date_set_dmy (date, 11, 3, 2008);
 #else
-  g_date_set_time_t (date, time (NULL)); 
+  g_date_set_time_t (date, time (NULL));
 #endif
   return date;
 }
@@ -190,19 +201,19 @@ match_next (const char  *input, GMatchInfo *info)
   char *s;
   GDateDay current_day, next_day;
   int add_days;
-  
+
   s = g_match_info_fetch_named (info, "day");
   next_day = convert_str_weekday (s);
   g_free (s);
   if (next_day == G_DATE_BAD_WEEKDAY)
     return NULL;
-  
+
   date = today ();
   current_day = g_date_get_weekday (date);
   add_days = 7 + next_day - current_day;
 
   g_date_add_days (date, add_days);
-  
+
   return date;
 }
 
@@ -213,13 +224,13 @@ match_this (const char *input, GMatchInfo *info)
   char *s;
   GDateDay today_day;
   GDateWeekday target_day;
-  
+
   s = g_match_info_fetch_named (info, "day");
   target_day = convert_str_weekday (s);
   g_free (s);
   if (target_day == G_DATE_BAD_WEEKDAY)
     return NULL;
-    
+
   date = today ();
   today_day = g_date_get_day (date);
   g_date_subtract_days (date, g_date_get_weekday (date));
@@ -227,7 +238,7 @@ match_this (const char *input, GMatchInfo *info)
 
   if (g_date_get_day (date) <= today_day)
     g_date_add_days (date, 7);
-  
+
   return date;
 }
 
@@ -250,14 +261,14 @@ match_date (const char *input, GMatchInfo *info)
     return NULL;
   month = atoi (month_s);
   g_free (month_s);
-  
+
   if (!g_date_valid_day (day) || !g_date_valid_month (month))
     return NULL;
 
   date = today ();
   g_date_set_day (date, day);
   g_date_set_month (date, month);
-  
+
   return date;
 }
 
@@ -283,7 +294,7 @@ match_day (const char *input, GMatchInfo *info)
 
   date = today ();
   g_date_set_day (date, day);
-  
+
   return date;
 }
 
@@ -311,33 +322,24 @@ match_day_month (const char *input, GMatchInfo *info)
   date = today ();
   g_date_set_day (date, day);
   g_date_set_month (date, month);
-  
+
   return date;
 }
 
 static GDate *
 match_digits (const char *input, GMatchInfo *info)
 {
-  char *s, *p;
-  struct tm tm = {0,};
   GDate *date;
+  char *s;
 
   s = g_match_info_fetch_named (info, "digits");
-  p = strptime (s, "%x", &tm);
-  g_free (s);
-  
-  if (p == NULL) {
+  if (!s)
     return NULL;
-  }
 
   date = g_date_new ();
-  if (tm.tm_mday)
-    g_date_set_day (date, tm.tm_mday);
-  if (tm.tm_mon)
-    g_date_set_month (date, tm.tm_mon + 1);
-  if (tm.tm_year)
-    g_date_set_year (date, tm.tm_year + 1900);
-  
+  g_date_set_parse (date, s);
+  g_free (s);
+
   if (g_date_valid (date)) {
     return date;
   } else {
@@ -346,6 +348,78 @@ match_digits (const char *input, GMatchInfo *info)
   }
 }
 
+static GDate *
+match_week (const char *input, GMatchInfo *info)
+{
+  GDate *date;
+  char *week_s;
+  int week, this_week;
+
+  week_s = g_match_info_fetch_named (info, "week");
+  if (!week_s)
+    return NULL;
+  week = atoi (week_s);
+  g_free (week_s);
+
+  if (week < 1 || week > 53)
+    return NULL;
+
+  date = today ();
+  this_week = g_date_get_iso8601_week_of_year (date);
+
+  /* Go to the Monday in week 1 */
+  g_date_set_day (date, 1);
+  g_date_set_month (date, G_DATE_JANUARY);
+  if (week <= this_week)
+    g_date_add_years (date, 1);
+  g_date_subtract_days (date, g_date_get_weekday (date) - G_DATE_MONDAY);
+
+  g_date_add_days (date, (week - 1) * 7);
+
+  return date;
+}
+
+static GDate *
+match_days (const char *input, GMatchInfo *info)
+{
+  GDate *date;
+  char *days_s;
+  int days;
+
+  days_s = g_match_info_fetch_named (info, "days");
+  if (!days_s)
+    return NULL;
+  days = atoi (days_s);
+  g_free (days_s);
+
+  if (days < 1)
+    return NULL;
+
+  date = today ();
+  g_date_add_days (date, days);
+  return date;
+}
+
+static GDate *
+match_weeks (const char *input, GMatchInfo *info)
+{
+  GDate *date;
+  char *weeks_s;
+  int weeks;
+
+  weeks_s = g_match_info_fetch_named (info, "weeks");
+  if (!weeks_s)
+    return NULL;
+  weeks = atoi (weeks_s);
+  g_free (weeks_s);
+
+  if (weeks < 1)
+    return NULL;
+
+  date = today ();
+  g_date_add_days (date, weeks * 7);
+  return date;
+}
 
 static Matcher*
 matcher_new (const gchar *regex_str, MatchFunc func)
@@ -356,11 +430,11 @@ matcher_new (const gchar *regex_str, MatchFunc func)
 
   g_assert (regex_str);
   g_assert (func);
-  
+
   regex = g_regex_new (regex_str,
 		       G_REGEX_CASELESS | G_REGEX_OPTIMIZE, 0,
 		       &error);
-  
+
   if (error) {
     g_warning ("Cannot parse regex '%s': %s", regex_str, error->message);
     g_error_free (error);
@@ -369,7 +443,7 @@ matcher_new (const gchar *regex_str, MatchFunc func)
 
   m = g_slice_new (Matcher);
   m->regex = regex;
-  m->func = func;  
+  m->func = func;
   return m;
 }
 
@@ -382,7 +456,14 @@ init_matchers (void)
 
   init_days ();
   init_months ();
-  
+
+  /*
+    Translators: WARNING: scary strings ahead.  If you don't understand regular
+    expressions then skip these, otherwise
+    http://www.burtonini.com/blog/computers/tasks-2008-10-01-21-17 has some
+    details about how this works.
+  */
+
   /* Translators: just translate "today" */
   m = matcher_new (_("^(?<task>.+) today$"), match_today);
   if (m) matchers = g_list_prepend (matchers, m);
@@ -403,6 +484,8 @@ init_matchers (void)
   g_free (s);
   if (m) matchers = g_list_prepend (matchers, m);
 
+  /* Translators: just translate this|on|by|due to contain relevant words in
+     your language, moving the day block is required. */
   m = matcher_new (_("^(?<task>.+) (this|on|by|due) (?<day>\\w+)$"),  match_this);
   if (m) matchers = g_list_prepend (matchers, m);
 
@@ -411,18 +494,44 @@ init_matchers (void)
   m = matcher_new (_("^(?<task>.+) (?:by|due|on) (?<day>\\d{1,2})/(?<month>\\d{1,2})$"),  match_date);
   if (m) matchers = g_list_prepend (matchers, m);
 
+  /* Translators: just translate on|by|due and st|nd|rd|th to contain relevant
+     words in your language. */
   m = matcher_new (_("^(?<task>.+) (?:by|due|on)? (?<day>\\d{1,2})(?:st|nd|rd|th)?$"),  match_day);
   if (m) matchers = g_list_prepend (matchers, m);
 
+  /* Translators: just translate on|by|due and st|nd|rd|th to contain relevant
+     words in your language, and move the month block if required. */
   m = matcher_new (_("^(?<task>.+) (?:by|due|on)? (?<day>\\d{1,2})(?:st|nd|rd|th)? (?<month>\\w+)$"),  match_day_month);
   if (m) matchers = g_list_prepend (matchers, m);
 
+  /* Translators: just translate on|by|due and st|nd|rd|th to contain relevant
+     words in your language, and move the month block if required. */
   m = matcher_new (_("^(?<task>.+) (?:by|due|on)? (?<month>\\w+) (?<day>\\d{1,2})(?:st|nd|rd|th)?$"),  match_day_month);
   if (m) matchers = g_list_prepend (matchers, m);
 
+  /* Translators: just translate this|on|by|due to contain relevant words in
+     your language */
   m = matcher_new (_("^(?<task>.+) (?:by|due|on)? (?<digits>[\\d/-]+)$"), match_digits);
   if (m) matchers = g_list_prepend (matchers, m);
-  
+
+  /* Translators: just translate this|on|by|due to contain relevant words in
+     your language and replace the w with whatever shortcut indicated "week". In
+     English, w34 means "week 34". */
+  m = matcher_new (_("^(?<task>.+) (?:by|due|on)? w(?<week>\\d{1,2})$"), match_week);
+  if (m) matchers = g_list_prepend (matchers, m);
+
+  /* Translators: just translate in ... days to contain relevant words in your
+     language. Remember the trailing ? is part of the regular expression to be
+     translated.*/
+  m = matcher_new (_("^(?<task>.+) in (?<days>\\d+) days?$"), match_days);
+  if (m) matchers = g_list_prepend (matchers, m);
+
+  /* Translators: just translate in ... weeksto contain relevant words in your
+     language. Remember the trailing ? is part of the regular expression to be
+     translated.*/
+  m = matcher_new (_("^(?<task>.+) in (?<weeks>\\d+) weeks?$"), match_weeks);
+  if (m) matchers = g_list_prepend (matchers, m);
+
   return g_list_reverse (matchers);
 }
 
@@ -435,10 +544,10 @@ koto_date_parse (const char *s, char **out_s)
 #if DEBUG
   g_printerr ("Testing '%s'\n", s);
 #endif
-  
+
   if (matchers == NULL)
     matchers = init_matchers ();
-  
+
   for (l = matchers; l != NULL; l = l->next) {
     Matcher *m = l->data;
     GMatchInfo *info;
@@ -451,12 +560,12 @@ koto_date_parse (const char *s, char **out_s)
       date = m->func (s, info);
       if (!date)
 	goto next;
-      
+
       if (!g_date_valid (date)) {
 	g_date_free (date);
 	goto next;
       }
-      
+
       if (out_s) {
 	*out_s = g_match_info_fetch_named (info, "task");
 #if DEBUG
@@ -514,14 +623,14 @@ main (int argc, char **argv)
   char *s = NULL;
 
   /*
-   * today() returns 11th March 2008 when running the test case.
+   * today() returns Tuesday 11th March 2008 when running the test case.
    */
-  
+
   /* Set the locale to C so we can do string comparisons easily */
   setlocale (LC_ALL, "C");
 
   /* Sanity checks for non-date strings */
-  
+
   TEST_PARSER_FAIL ("foo");
   TEST_PARSER_FAIL ("foo and bar");
 
@@ -535,7 +644,7 @@ main (int argc, char **argv)
 
   g_date_subtract_days (expected, 2);
   TEST_PARSER ("Foo Yesterday", "Foo");
-  
+
   /* On 21st, etc */
 
   g_date_set_dmy (expected, 1, 3, 2008);
@@ -552,7 +661,7 @@ main (int argc, char **argv)
 
   TEST_PARSER_FAIL ("foo by 999th");
   TEST_PARSER_FAIL ("foo on 32th");
-  
+
   /* On 1st June, etc */
 
   g_date_set_dmy (expected, 1, 4, 2008);
@@ -571,7 +680,7 @@ main (int argc, char **argv)
   TEST_PARSER ("foo by jan 3rd", "foo");
 
   /* This/etc [weekday] */
-  
+
   g_date_set_dmy (expected, 12, 3, 2008);
   TEST_PARSER ("foo this wednesday", "foo");
   TEST_PARSER ("foo this wed", "foo");
@@ -604,7 +713,7 @@ main (int argc, char **argv)
   TEST_PARSER_FAIL ("foo due fish");
   TEST_PARSER_FAIL ("foo by fish");
   TEST_PARSER_FAIL ("fish this flob");
-  
+
   /* Next [weekday] */
 
   g_date_set_dmy (expected, 17, 3, 2008);
@@ -629,15 +738,17 @@ main (int argc, char **argv)
   TEST_PARSER ("foo next sunday", "foo");
 
   TEST_PARSER_FAIL ("foo next flobday");
-  
-  /* Digits */
-  /* Remember we're in in the C locale */
+
+  /* Digits.  Remember we're in in the C locale so the dates are m/d/y */
 
   g_date_set_dmy (expected, 17, 3, 2008);
   TEST_PARSER ("foo on 3/17/08", "foo");
   g_date_set_dmy (expected, 29, 2, 2008);
-  TEST_PARSER ("foo on 2/29/08", "foo");
-  
+  TEST_PARSER ("foo on 2/29/8", "foo");
+  g_date_set_dmy (expected, 13, 4, 2008);
+  TEST_PARSER ("foo on 4-13-08", "foo");
+  g_date_set_dmy (expected, 1, 1, 2008);
+  TEST_PARSER ("foo on 1-1-8", "foo");
   /* All out of range */
   TEST_PARSER_FAIL ("foo on 99/99/99");
   /* Invalid month */
@@ -647,8 +758,80 @@ main (int argc, char **argv)
   TEST_PARSER_FAIL ("foo on 2/29/07");
   TEST_PARSER_FAIL ("foo on 4/31/07");
 
+  /* Week numbers */
+
+  g_date_set_dmy (expected, 17, 3, 2008);
+  TEST_PARSER ("foo on w12", "foo");
+
+  g_date_set_dmy (expected, 26, 5, 2008);
+  TEST_PARSER ("foo on w22", "foo");
+
+  g_date_set_dmy (expected, 22, 12, 2008);
+  TEST_PARSER ("foo on w52", "foo");
+
+  g_date_set_dmy (expected, 29, 12, 2008);
+  TEST_PARSER ("foo on w1", "foo");
+
+  g_date_set_dmy (expected, 5, 1, 2009);
+  TEST_PARSER ("foo on w2", "foo");
+
+  g_date_set_dmy (expected, 9, 3, 2009);
+  TEST_PARSER ("foo on w11", "foo");
+
+  /* In x days */
+
+  g_date_set_dmy (expected, 12, 3, 2008);
+  TEST_PARSER ("foo in 1 day", "foo");
+
+  g_date_set_dmy (expected, 13, 3, 2008);
+  TEST_PARSER ("foo in 2 days", "foo");
+
+  g_date_set_dmy (expected, 10, 4, 2008);
+  TEST_PARSER ("foo in 30 days", "foo");
+
+  g_date_set_dmy (expected, 11, 3, 2009);
+  TEST_PARSER ("foo in 365 days", "foo");
+
+  TEST_PARSER_FAIL ("foo in -10 days");
+  TEST_PARSER_FAIL ("foo in 0 days");
+
+  /* In x weeks */
+
+  g_date_set_dmy (expected, 18, 3, 2008);
+  TEST_PARSER ("foo in 1 week", "foo");
+
+  g_date_set_dmy (expected, 25, 3, 2008);
+  TEST_PARSER ("foo in 2 weeks", "foo");
+
+  g_date_set_dmy (expected, 20, 5, 2008);
+  TEST_PARSER ("foo in 10 weeks", "foo");
+
+  TEST_PARSER_FAIL ("foo in -10 weeks");
+  TEST_PARSER_FAIL ("foo in 0 weeks");
+
   g_date_free (expected);
-  
+
   return 0;
 }
-#endif
+#endif /* BUILD_TESTS */
+
+
+#else /* HAVE_DECL_G_REGEX_NEW */
+
+/* Stub implementation for when we don't have GRegex available. */
+
+GDate *
+koto_date_parse (const char *s, char **out_s)
+{
+  return NULL;
+}
+
+#if BUILD_TESTS
+int
+main (int argc, char **argv)
+{
+  return 0;
+}
+#endif /* BUILD_TESTS */
+
+#endif /* HAVE_DECL_G_REGEX_NEW */
