@@ -58,18 +58,17 @@ response_cb (GtkDialog *dialog, int response)
   KotoTaskEditorDialogPrivate *priv = GET_PRIVATE (dialog);
   GtkWidget *confirm;
   KotoUndoContext *undo = NULL;
-  
+
   if (!priv->task) {
     return;
   }
-  
-  switch (response) {
-  case KOTO_TASK_EDITOR_DIALOG_RESPONSE_DELETE:
+
+  if (response == KOTO_TASK_EDITOR_DIALOG_RESPONSE_DELETE) {
     confirm = gtk_message_dialog_new (GTK_WINDOW (dialog),
-                                     GTK_DIALOG_DESTROY_WITH_PARENT,
-                                     GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
-                                     _("Are you sure you want to delete \"%s\"?"),
-                                     icalcomponent_get_summary (priv->task->comp));
+                                      GTK_DIALOG_DESTROY_WITH_PARENT,
+                                      GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
+                                      _("Are you sure you want to delete \"%s\"?"),
+                                      icalcomponent_get_summary (priv->task->comp));
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (confirm),
                                               _("If you delete an item, it is permanently lost."));
     gtk_dialog_add_buttons (GTK_DIALOG (confirm),
@@ -77,7 +76,7 @@ response_cb (GtkDialog *dialog, int response)
                             GTK_STOCK_DELETE, GTK_RESPONSE_ACCEPT,
                             NULL);
     gtk_dialog_set_default_response (GTK_DIALOG (confirm), GTK_RESPONSE_ACCEPT);
-    
+
     if (gtk_dialog_run (GTK_DIALOG (confirm)) == GTK_RESPONSE_ACCEPT) {
       if (priv->undo_manager)
         undo = koto_undo_manager_context_begin_formatted (priv->undo_manager,
@@ -93,18 +92,18 @@ response_cb (GtkDialog *dialog, int response)
       /* Stop the response signal from getting anywhere. */
       g_signal_stop_emission_by_name (dialog, "response");
     }
-    
+
     gtk_widget_destroy (confirm);
-  default:
+  } else {
     if (koto_task_editor_dialog_is_dirty (KOTO_TASK_EDITOR_DIALOG (dialog))) {
       if (priv->undo_manager)
         undo = koto_undo_manager_context_begin_formatted (priv->undo_manager,
                                                           _("Edit Task %s"),
                                                           icalcomponent_get_summary (priv->task->comp),
                                                           NULL);
-      
+
       koto_action_modify_task (priv->cal, priv->task, priv->old_comp, undo);
-      
+
       if (priv->undo_manager)
         koto_undo_manager_context_end (priv->undo_manager, undo);
     }
@@ -117,7 +116,7 @@ koto_task_editor_dialog_get_property (GObject *object, guint property_id,
 {
   KotoTaskEditorDialog *editor = KOTO_TASK_EDITOR_DIALOG (object);
   KotoTaskEditorDialogPrivate *priv = GET_PRIVATE (editor);
-  
+
   switch (property_id) {
   case PROP_TASK:
     g_value_set_boxed (value, priv->task);
@@ -176,12 +175,12 @@ static void
 koto_task_editor_dialog_dispose (GObject *object)
 {
   KotoTaskEditorDialogPrivate *priv = GET_PRIVATE (object);
-  
+
   if (priv->cal) {
     g_object_unref (priv->cal);
     priv->cal = NULL;
   }
-  
+
   if (priv->task) {
     koto_task_unref (priv->task);
     priv->task = NULL;
@@ -191,7 +190,7 @@ koto_task_editor_dialog_dispose (GObject *object)
     g_object_unref (priv->undo_manager);
     priv->undo_manager = NULL;
   }
-  
+
   if (G_OBJECT_CLASS (koto_task_editor_dialog_parent_class)->dispose)
     G_OBJECT_CLASS (koto_task_editor_dialog_parent_class)->dispose (object);
 }
@@ -199,6 +198,10 @@ koto_task_editor_dialog_dispose (GObject *object)
 static void
 koto_task_editor_dialog_finalize (GObject *object)
 {
+  KotoTaskEditorDialogPrivate *priv = GET_PRIVATE (object);
+
+  icalcomponent_free (priv->old_comp);
+
   G_OBJECT_CLASS (koto_task_editor_dialog_parent_class)->finalize (object);
 }
 
@@ -330,9 +333,9 @@ koto_task_editor_dialog_set_task (KotoTaskEditorDialog *dialog, KotoTask *task)
   if (priv->old_comp)
     icalcomponent_free (priv->old_comp);
   priv->old_comp = icalcomponent_new_clone (task->comp);
-  
+
   g_object_set (priv->editor, "task", priv->task, NULL);
-  
+
   koto_field_editor_set (priv->desc, task);
   if (icalcomponent_get_description (priv->task->comp)) {
     gtk_label_set_markup (GTK_LABEL (priv->desc_label), _("<b>_Notes</b>"));
@@ -340,7 +343,7 @@ koto_task_editor_dialog_set_task (KotoTaskEditorDialog *dialog, KotoTask *task)
     gtk_label_set_text (GTK_LABEL (priv->desc_label), _("_Notes"));
   }
   gtk_label_set_use_underline (GTK_LABEL (priv->desc_label), TRUE);
-  
+
   title = g_strdup_printf (_("%s - Tasks"), icalcomponent_get_summary (task->comp));
   gtk_window_set_title (GTK_WINDOW (dialog), title);
   g_free (title);
