@@ -35,7 +35,7 @@ struct _KotoTaskStorePrivate
 {
   ECal *cal;
   ECalView *view;
-  guint sig_added, sig_modified, sig_removed;
+  guint sig_added, sig_modified, sig_removed, sig_done;
   GHashTable *uid_hash;
 };
 
@@ -44,6 +44,13 @@ enum {
   PROP_CALVIEW,
 };
 
+enum {
+  SIGNAL_LOADED,
+  NUM_SIGNALS
+};
+
+static guint signals[NUM_SIGNALS];
+
 /*
  * Private methods
  */
@@ -71,14 +78,14 @@ sorter_cb (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_da
     res = done_a < done_b ? -1 : 1;
     goto done;
   }
-  
+
   if (weight_a != weight_b) {
     res = weight_a < weight_b ? -1 : 1;
     goto done;
   }
-  
+
   res = g_utf8_collate (summary_a ?: "", summary_b ?: "");
-  
+
  done:
   g_free (summary_a);
   g_free (summary_b);
@@ -137,7 +144,7 @@ update_row (KotoTaskStore *store, icalcomponent *ical, gboolean insert, GtkTreeI
   g_assert (KOTO_IS_TASK_STORE (store));
   g_assert (ical);
   g_assert (iter);
-  
+
   status = icalcomponent_get_status (ical);
   due_time = icalcomponent_get_due (ical);
   if (!icaltime_is_null_time (due_time)) {
@@ -146,7 +153,7 @@ update_row (KotoTaskStore *store, icalcomponent *ical, gboolean insert, GtkTreeI
                        icaltime_as_timet_with_zone (due_time,
                                                     icaltimezone_get_utc_timezone ()));
   }
-  
+
   task = koto_task_new (icalcomponent_new_clone (ical));
 
   if (insert) {
@@ -170,7 +177,7 @@ update_row (KotoTaskStore *store, icalcomponent *ical, gboolean insert, GtkTreeI
                         COLUMN_URL, ical_util_get_url (ical),
                         -1);
   }
-  
+
   if (due_date)
     g_date_free (due_date);
   koto_task_unref (task);
@@ -195,7 +202,7 @@ on_objects_added (ECalView *view, GList *objects, KotoTaskStore *store)
     g_hash_table_insert (priv->uid_hash,
                          g_strdup (icalcomponent_get_uid (ical)),
                          gtk_tree_iter_copy (&iter));
-    
+
   }
 }
 
@@ -247,6 +254,18 @@ on_objects_removed (ECalView *view, GList *uids, KotoTaskStore *store) {
   }
 }
 
+static void
+on_view_done (ECalView *view, int status, KotoTaskStore *store)
+{
+  KotoTaskStorePrivate *priv = GET_PRIVATE (store);
+
+  g_signal_emit (store, signals[SIGNAL_LOADED], 0);
+
+  /* This is a one-shot signal */
+  g_signal_handler_disconnect (priv->view, priv->sig_done);
+  priv->sig_done = 0;
+}
+
 /*
  * Object methods
  */
@@ -295,7 +314,7 @@ koto_task_store_finalize (GObject *object)
   KotoTaskStorePrivate *priv = GET_PRIVATE (object);
 
   g_hash_table_destroy (priv->uid_hash);
-  
+
   G_OBJECT_CLASS (koto_task_store_parent_class)->finalize (object);
 }
 
@@ -316,6 +335,13 @@ koto_task_store_class_init (KotoTaskStoreClass *klass)
                                                         E_TYPE_CAL_VIEW,
                                                         G_PARAM_READWRITE |
                                                         G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK));
+
+  signals[SIGNAL_LOADED] = g_signal_new ("loaded",
+                                  G_TYPE_FROM_CLASS (klass),
+                                  G_SIGNAL_RUN_FIRST,
+                                  0, NULL, NULL,
+                                  g_cclosure_marshal_VOID__VOID,
+                                  G_TYPE_NONE, 0);
 }
 
 static void
@@ -373,14 +399,16 @@ koto_task_store_set_view (KotoTaskStore *store, ECalView *view)
     g_signal_handler_disconnect (priv->view, priv->sig_added);
     g_signal_handler_disconnect (priv->view, priv->sig_modified);
     g_signal_handler_disconnect (priv->view, priv->sig_removed);
-    priv->sig_added = priv->sig_modified = priv->sig_removed = 0;
-    
+    if (priv->sig_done)
+      g_signal_handler_disconnect (priv->view, priv->sig_done);
+    priv->sig_added = priv->sig_modified = priv->sig_removed = priv->sig_done = 0;
+
     g_object_unref (priv->view);
     g_object_unref (priv->cal);
     priv->view = NULL;
     priv->cal = NULL;
   }
-  
+
   if (view) {
     priv->view = g_object_ref (view);
     priv->cal = g_object_get_data (G_OBJECT (view), "koto-ecal");
@@ -389,6 +417,7 @@ koto_task_store_set_view (KotoTaskStore *store, ECalView *view)
     priv->sig_added = g_signal_connect (priv->view, "objects-added", G_CALLBACK (on_objects_added), store);
     priv->sig_modified = g_signal_connect (priv->view, "objects-modified", G_CALLBACK (on_objects_modified), store);
     priv->sig_removed = g_signal_connect (priv->view, "objects-removed", G_CALLBACK (on_objects_removed), store);
+    priv->sig_done = g_signal_connect (priv->view, "view-done", G_CALLBACK (on_view_done), store);
   }
 }
 
@@ -403,13 +432,13 @@ koto_task_store_set_done (KotoTaskStore *store, GtkTreeIter *iter, gboolean done
   g_return_if_fail (iter);
 
   priv = GET_PRIVATE (store);
-  
+
   gtk_tree_model_get (GTK_TREE_MODEL (store), iter, COLUMN_TASK, &task, -1);
-  
+
   old_comp = icalcomponent_new_clone (task->comp);
-  
+
   icalcomponent_set_status (task->comp, done ? ICAL_STATUS_COMPLETED : ICAL_STATUS_NONE);
-  
+
   gtk_list_store_set (GTK_LIST_STORE (store), iter, COLUMN_DONE, done, -1);
 
   koto_action_modify_task (priv->cal, task, old_comp, undo);
