@@ -48,6 +48,11 @@
 
 #if WITH_UNIQUE
 #include <unique/unique.h>
+
+enum {
+  COMMAND_ACTIVATE = UNIQUE_ACTIVATE,
+  COMMAND_EDIT = 1
+};
 #endif
 
 #include "ui.h"
@@ -64,23 +69,20 @@ static KotoUndoManager *undo_manager;
 static gboolean
 select_uid (char *uid)
 {
-  GtkTreeSelection *selection;
   GtkTreeIter iter, real_iter;
   GtkTreePath *path;
 
   g_assert (uid);
-  
+
   if (koto_task_store_get_iter_for_uid (KOTO_TASK_STORE (task_store), uid, &iter)) {
-    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
-    
     gtk_tree_model_filter_convert_child_iter_to_iter (GTK_TREE_MODEL_FILTER (filter),
                                                       &real_iter, &iter);
-    
+
     path = gtk_tree_model_get_path (filter, &real_iter);
     gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), path, NULL, FALSE);
     gtk_tree_path_free (path);
   }
-  
+
   g_free (uid);
   return FALSE;
 }
@@ -126,7 +128,7 @@ create_component (const char *text, KotoGroup *default_group)
     text = g_utf8_next_char (text);
     skip_whitespace (&text);
   }
-  
+
   /* If the task starts with @foo, put it in the "foo" group. */
   if (text[0] == '@') {
     char *end, *guess;
@@ -141,14 +143,14 @@ create_component (const char *text, KotoGroup *default_group)
       /* Try and find a matching group */
       group = koto_group_store_match_group
         (KOTO_GROUP_STORE (group_selector_store), guess);
-      
+
       if (group)
         g_free (guess);
       else
         group = guess;
     }
   }
-  
+
   /* Parse any date hints in the text */
   date = koto_date_parse (text, &parsed_text);
   if (date) {
@@ -161,7 +163,7 @@ create_component (const char *text, KotoGroup *default_group)
   icalcomponent_add_property (comp,
 			      icalproperty_new_summary (parsed_text ?: text));
   g_free (parsed_text);
-  
+
   /* If a group wasn't specified in the task, use the current group */
   if (group == NULL) {
     if (default_group &&
@@ -170,7 +172,7 @@ create_component (const char *text, KotoGroup *default_group)
       group = g_strdup (koto_group_get_name (default_group));
     }
   }
-  
+
   if (group) {
     icalcomponent_add_property (comp, icalproperty_new_categories (group));
     g_free (group);
@@ -189,7 +191,7 @@ on_new_clicked (GtkButton *button)
   KotoUndoContext *ctxt;
 
   text = gtk_entry_get_text (GTK_ENTRY (new_entry));
-  
+
   if (!text || text[0] == '\0') {
     g_warning ("Got clicked with empty text");
     return;
@@ -205,7 +207,7 @@ on_new_clicked (GtkButton *button)
                                                     NULL);
 
   koto_action_create_task (cal, comp, &uid, ctxt);
-  
+
   koto_undo_manager_context_end (undo_manager, ctxt);
 
   koto_hint_entry_clear (KOTO_HINT_ENTRY (new_entry));
@@ -220,7 +222,7 @@ on_new_clicked (GtkButton *button)
   }
 }
 
-/* 
+/*
  * Used to enable/disable the new task button depending on the contents of the
  * entry.
  */
@@ -251,18 +253,18 @@ edit_task (KotoTask *task)
     dialog = koto_task_editor_dialog_new ();
     gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
     g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
-    
+
     g_object_set (dialog,
                   "cal", cal,
                   "groups", group_selector_store,
                   "task", task,
                   "undo-manager", undo_manager,
                   NULL);
-    
+
     window_bind_state (GTK_WINDOW (dialog), "task-editor");
 
     gtk_widget_show (dialog);
-    
+
     g_hash_table_insert (edit_dialog_hash, task, dialog);
     g_object_weak_ref (G_OBJECT (dialog), edit_dialog_weak_notify, koto_task_ref (task));
   }
@@ -296,9 +298,21 @@ static void
 on_selection_changed (GtkTreeSelection *selection, gpointer user_data)
 {
   GtkActionGroup *actions = user_data;
+  GtkAction *action = NULL;
+  gboolean complete = FALSE;
+  GtkTreeModel *model = NULL;
+  GtkTreeIter iter;
 
+  /* Enable or disable the group based on the number of rows selected */
   gtk_action_group_set_sensitive
     (actions, gtk_tree_selection_count_selected_rows (selection) != 0);
+
+  /* Set the state of the complete action based on the current task */
+  action = gtk_action_group_get_action (actions, "CompleteTask");
+  if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+    gtk_tree_model_get (model, &iter, COLUMN_DONE, &complete, -1);
+  }
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), complete);
 }
 
 /*
@@ -323,27 +337,29 @@ on_edit_task_action (GtkAction *action, gpointer user_data)
     g_warning ("No task selected, EditTask should be disabled");
     return;
   }
-  
+
   edit_task (task);
 
   koto_task_unref (task);
 }
 
 static void
-on_complete_task_action (GtkAction *action, gpointer user_data)
+on_complete_task_action (GtkToggleAction *action, gpointer user_data)
 {
   GtkTreeIter real_iter, iter;
   KotoUndoContext *undo;
+  gboolean state;
 
   if (!koto_task_view_get_selected_iter (KOTO_TASK_VIEW (treeview), &iter)) {
-    g_warning ("No task selected, CompleteTask should be disabled");
     return;
   }
 
+  state = gtk_toggle_action_get_active (action);
+
   gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (filter), &real_iter, &iter);
-  
+
   undo = koto_undo_manager_context_begin (undo_manager, _("Complete Task"));
-  koto_task_store_set_done (KOTO_TASK_STORE (task_store), &real_iter, TRUE, undo);
+  koto_task_store_set_done (KOTO_TASK_STORE (task_store), &real_iter, state, undo);
   koto_undo_manager_context_end (undo_manager, undo);
 }
 
@@ -361,14 +377,14 @@ on_delete_task_action (GtkAction *action, gpointer user_data)
     g_warning ("No task selected, DeleteTask should be disabled");
     return;
   }
-  
+
   ctxt = koto_undo_manager_context_begin_formatted (undo_manager,
                                                     _("Delete Task %s"),
                                                     icalcomponent_get_summary (task->comp),
                                                     NULL);
   koto_action_delete_task (cal, task, ctxt);
   koto_undo_manager_context_end (undo_manager, ctxt);
-  
+
   koto_task_unref (task);
 }
 
@@ -387,13 +403,13 @@ purge_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpoint
                       COLUMN_DONE, &done,
                       COLUMN_TASK, &task,
                       -1);
-  
+
   if (done) {
     koto_action_delete_task (cal, task, ctxt);
   }
 
   koto_task_unref (task);
-  
+
   return FALSE;
 }
 
@@ -406,9 +422,9 @@ on_purge_action (GtkAction *action, gpointer user_data)
   KotoUndoContext *ctxt;
 
   ctxt = koto_undo_manager_context_begin (undo_manager, _("Remove Completed"));
-  
+
   gtk_tree_model_foreach (task_store, purge_foreach, ctxt);
-  
+
   koto_undo_manager_context_end (undo_manager, ctxt);
 }
 
@@ -457,7 +473,7 @@ on_about_action (GtkAction *action, gpointer user_data)
                          NULL);
 }
 
-static const GtkActionEntry actions[] = 
+static const GtkActionEntry actions[] =
 {
   /* Action name, stock ID, label, accelerator, tooltip, callback */
   { "TasksMenu", NULL, N_("_Task") },
@@ -471,39 +487,71 @@ static const GtkActionEntry actions[] =
   { "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (on_about_action) },
 };
 
-static const GtkActionEntry task_actions[] = 
+static const GtkActionEntry task_actions[] =
 {
   { "EditTask", GTK_STOCK_EDIT, N_("Edit..."), "<control>e", NULL, G_CALLBACK (on_edit_task_action) },
-  /* TODO: turn this action into a toggle action */
-  { "CompleteTask", NULL, N_("_Mark Complete"), "<control>d", NULL, G_CALLBACK (on_complete_task_action) },
   { "DeleteTask", GTK_STOCK_DELETE, NULL, "<control>Delete", NULL, G_CALLBACK (on_delete_task_action) },
 };
 
+static const GtkToggleActionEntry task_toggle_actions[] =
+{
+  { "CompleteTask", NULL, N_("_Mark Complete"), "<control>d", NULL, G_CALLBACK (on_complete_task_action), FALSE},
+};
+
+/*
+ * Callback when the store has loaded if we have been asked to edit a task.
+ */
+static void
+edit_loaded (KotoTaskStore *store, gpointer user_data)
+{
+  GtkTreeIter iter;
+  KotoTask *task = NULL;
+  const char *edit_uid = user_data;
+
+  if (koto_task_store_get_iter_for_uid (store, edit_uid, &iter)) {
+    gtk_tree_model_get ((GtkTreeModel*)store, &iter, COLUMN_TASK, &task, -1);
+    edit_task (task);
+    koto_task_unref (task);
+  }
+
+  g_signal_handlers_disconnect_by_func (store, edit_loaded, user_data);
+}
+
 void
 koto_platform_open_url (const char *url)
 {
+#if HAVE_DECL_GTK_SHOW_URI
+  GError *error = NULL;
+
+  /* TODO: pass screen and timestamp */
+  if (!gtk_show_uri (NULL, url, GDK_CURRENT_TIME, &error)) {
+    g_warning ("Cannot open URL: %s", error->message);
+    g_error_free (error);
+  }
+#else
   char cmd[1024];
 
-  g_snprintf (cmd, sizeof (cmd), "gnome-open \"%s\"", url);  
+  g_snprintf (cmd, sizeof (cmd), "gnome-open \"%s\"", url);
   if (g_spawn_command_line_async (cmd, NULL))
     return;
 
-  g_snprintf (cmd, sizeof (cmd), "xdg-open \"%s\"", url);  
+  g_snprintf (cmd, sizeof (cmd), "xdg-open \"%s\"", url);
   if (g_spawn_command_line_async (cmd, NULL))
     return;
 
-  g_snprintf (cmd, sizeof (cmd), "firefox \"%s\"", url);  
+  g_snprintf (cmd, sizeof (cmd), "firefox \"%s\"", url);
   if (g_spawn_command_line_async (cmd, NULL))
     return;
 
   g_warning ("Cannot start gnome-open, xdg-open, or firefox");
+#endif
 }
 
 void
 koto_platform_edit_task (KotoTask *task)
 {
   g_return_if_fail (task);
-  
+
   edit_task (task);
 }
 
@@ -516,16 +564,29 @@ do_message_received (UniqueApp         *app,
                      gpointer           user_data)
 {
   switch (command) {
-  case UNIQUE_ACTIVATE:
+  case COMMAND_ACTIVATE:
     gtk_window_set_screen (GTK_WINDOW (window),
 			   unique_message_data_get_screen (message));
     gtk_window_present_with_time (GTK_WINDOW (window), time_);
     break;
+  case COMMAND_EDIT:
+    {
+      GtkTreeIter iter;
+      if (koto_task_store_get_iter_for_uid (KOTO_TASK_STORE (task_store),
+                                            unique_message_data_get_text (message),
+                                            &iter)) {
+        KotoTask *task;
+        gtk_tree_model_get (task_store, &iter, COLUMN_TASK, &task, -1);
+        edit_task (task);
+        koto_task_unref (task);
+      }
+    }
+    break;
   default:
     g_debug ("Unhandled unique command %d", command);
     break;
   }
-  
+
   return UNIQUE_RESPONSE_OK;
 }
 #endif
@@ -539,7 +600,12 @@ main (int argc, char **argv)
   GtkTreeSelection *selection;
   GtkActionGroup *action_group, *task_action_group;
   GtkUIManager *ui_manager;
-  
+  char *edit_uid = NULL;
+  GOptionEntry entries[] = {
+    { "edit", 'e', 0, G_OPTION_ARG_STRING, &edit_uid, "Edit item", "<UID>" },
+    { NULL }
+  };
+
 #if WITH_UNIQUE
   UniqueApp *app;
 #endif
@@ -552,20 +618,31 @@ main (int argc, char **argv)
 #endif
 
   g_set_application_name (_("Tasks"));
-  gtk_init (&argc, &argv);
+  gtk_init_with_args (&argc, &argv, _(" - a to do list"),
+                      entries, GETTEXT_PACKAGE, &error);
+
+  if (error) {
+    g_printerr ("%s\n", error->message);
+    g_error_free (error);
+    return EXIT_FAILURE;
+  }
 
 #if WITH_UNIQUE
-  app = unique_app_new ("org.pimlico-project.Tasks", NULL);
-
+  app = unique_app_new_with_commands ("org.pimlico-project.Tasks", NULL,
+                                      "edit", COMMAND_EDIT,
+                                      NULL);
   if (unique_app_is_running (app)) {
-    UniqueResponse response;
-
-    response = unique_app_send_message (app, UNIQUE_ACTIVATE, NULL);
-
-    gdk_notify_startup_complete ();
+    if (edit_uid == NULL) {
+      unique_app_send_message (app, COMMAND_ACTIVATE, NULL);
+    } else {
+      UniqueMessageData *message;
+      message = unique_message_data_new ();
+      unique_message_data_set_text (message, edit_uid, -1);
+      unique_app_send_message (app, COMMAND_EDIT, message);
+      unique_message_data_free (message);
+    }
 
     g_object_unref (app);
-
     return EXIT_SUCCESS;
   }
 #endif
@@ -577,18 +654,18 @@ main (int argc, char **argv)
   cal = e_cal_new_system_tasks ();
   if (!cal)
     g_error ("Cannot get system tasks");
-  
+
   if (!e_cal_open (cal, FALSE, &error))
     g_error("Cannot open calendar: %s", error->message);
 
   if (!e_cal_get_query (cal, "#t", &cal_view, &error))
-    g_error("Cannot get calendar view: %s", error->message);
+    g_error ("Cannot get calendar view: %s", error->message);
   /* TODO: nasty, should pass cal to the stores or add e_cal_view_get_cal() */
   g_object_set_data_full (G_OBJECT (cal_view), "koto-ecal", g_object_ref (cal), g_object_unref);
 
   /* Create the data stores */
   task_store = koto_task_store_new (cal_view);
-  
+
   group_filter_store = koto_group_store_new (cal_view);
   koto_group_store_add_group (KOTO_GROUP_STORE (group_filter_store), koto_all_group_new ());
   koto_group_store_add_group (KOTO_GROUP_STORE (group_filter_store), koto_meta_group_new (KOTO_META_GROUP_SEPERATOR, -99));
@@ -600,7 +677,7 @@ main (int argc, char **argv)
   koto_group_store_add_group (KOTO_GROUP_STORE (group_selector_store), koto_meta_group_new (KOTO_META_GROUP_SEPERATOR, -99));
   koto_group_store_add_group (KOTO_GROUP_STORE (group_selector_store), koto_meta_group_new (KOTO_META_GROUP_SEPERATOR, 99));
   koto_group_store_add_group (KOTO_GROUP_STORE (group_selector_store), koto_meta_group_new (KOTO_META_GROUP_NEW, 100));
-  
+
   /* Create the UI */
   gtk_window_set_default_icon_name ("tasks");
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -614,7 +691,7 @@ main (int argc, char **argv)
 #endif
 
   top_box = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (window), top_box); 
+  gtk_container_add (GTK_CONTAINER (window), top_box);
 
   action_group = gtk_action_group_new ("Actions");
   gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
@@ -629,6 +706,7 @@ main (int argc, char **argv)
   task_action_group = gtk_action_group_new ("Task Actions");
   gtk_action_group_set_translation_domain (task_action_group, GETTEXT_PACKAGE);
   gtk_action_group_add_actions (task_action_group, task_actions, G_N_ELEMENTS (task_actions), NULL);
+  gtk_action_group_add_toggle_actions (task_action_group, task_toggle_actions, G_N_ELEMENTS (task_toggle_actions), NULL);
 
   ui_manager = gtk_ui_manager_new ();
   gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
@@ -681,7 +759,7 @@ main (int argc, char **argv)
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
   g_signal_connect (selection, "changed", G_CALLBACK (on_selection_changed), task_action_group);
   on_selection_changed (selection, task_action_group);
-  
+
   hbox = gtk_hbox_new (FALSE, 4);
   new_entry = koto_hint_entry_new (_("New task..."));
   gtk_entry_set_activates_default (GTK_ENTRY (new_entry), TRUE);
@@ -709,6 +787,12 @@ main (int argc, char **argv)
                           GTK_TREE_MODEL (filter),
                           _("Tasks (%d)"));
 
+  /* If an edit UID was specified, connect a signal to open a dialog when we
+     find it */
+  if (edit_uid) {
+    g_signal_connect (task_store, "loaded", G_CALLBACK (edit_loaded), edit_uid);
+  }
+
   e_cal_view_start (cal_view);
 
   gtk_widget_show_all (window);
