@@ -18,4 +18,4 @@
 
 #include <gtk/gtkwindow.h>
 
-void window_bind_state (GtkWindow *window);
+void window_bind_state (GtkWindow *window, const char *name);
